@@ -148,7 +148,7 @@ done:
  * @font: a #PangoFont from a #PangoCairoFontMap
  *
  * Gets the #cairo_scaled_font_t used by @font.
- * The scaled font can be referenced and ketp using
+ * The scaled font can be referenced and kept using
  * cairo_scaled_font_reference().
  *
  * Return value: the #cairo_scaled_font_t used by @font
@@ -156,7 +156,7 @@ done:
  * Since: 1.18
  **/
 cairo_scaled_font_t *
-pango_cairo_font_get_scaled_font (PangoFont *cfont)
+pango_cairo_font_get_scaled_font (PangoCairoFont *cfont)
 {
   PangoCairoFontPrivate *cf_priv;
 
@@ -189,7 +189,7 @@ gboolean
 _pango_cairo_font_install (PangoFont *font,
 			   cairo_t   *cr)
 {
-  cairo_scaled_font_t *scaled_font = pango_cairo_font_get_scaled_font (font);
+  cairo_scaled_font_t *scaled_font = pango_cairo_font_get_scaled_font ((PangoCairoFont *)font);
 
   if (G_UNLIKELY (scaled_font == NULL || cairo_scaled_font_status (scaled_font) != CAIRO_STATUS_SUCCESS))
     return FALSE;
@@ -404,6 +404,7 @@ _pango_cairo_font_private_get_hex_box_info (PangoCairoFontPrivate *cf_priv)
     g_object_unref (context);
   }
 
+  pango_font_description_free (desc);
   cairo_font_options_destroy (font_options);
 
 
@@ -469,6 +470,7 @@ _pango_cairo_font_private_get_hex_box_info (PangoCairoFontPrivate *cf_priv)
        hbi->box_descent = HINT_Y (hbi->box_descent);
     }
 
+  cf_priv->hbi = hbi;
   return hbi;
 }
 
