@@ -50,7 +50,7 @@ set_color (PangoCairoRenderer *crenderer,
 	   PangoRenderPart     part)
 {
   PangoColor *color = pango_renderer_get_color ((PangoRenderer *) (crenderer), part);
-  
+
   if (color)
     cairo_set_source_rgb (crenderer->cr,
 			  color->red / 65535.,
@@ -69,7 +69,6 @@ _pango_cairo_renderer_draw_frame (PangoCairoRenderer *crenderer,
 {
   if (crenderer->do_path)
     {
-      cairo_t *cr = crenderer->cr;
       double d2 = line_width * .5, d = line_width;
 
       /* we draw an outer box in one winding direction and an inner one in the
@@ -79,18 +78,9 @@ _pango_cairo_renderer_draw_frame (PangoCairoRenderer *crenderer,
        */
 
       /* outer */
-      cairo_move_to (cr, x - d2, y - d2);
-      cairo_rel_line_to (cr, width + d, 0);
-      cairo_rel_line_to (cr, 0, height + d);
-      cairo_rel_line_to (cr, - (width + d), 0);
-      cairo_close_path (cr);
-
+      cairo_rectangle (crenderer->cr, x-d2, y-d2, width+d, height+d);
       /* inner */
-      cairo_move_to (cr, x + d2, y + d2);
-      cairo_rel_line_to (cr, 0, height - d);
-      cairo_rel_line_to (cr, width - d, 0);
-      cairo_rel_line_to (cr, 0, - (height - d));
-      cairo_close_path (cr);
+      cairo_rectangle (crenderer->cr, x+d2 + (width-d), y+d2, - (width-d), height-d);
     }
   else
     {
@@ -136,15 +126,15 @@ _pango_cairo_renderer_draw_unknown_glyph (PangoCairoRenderer *crenderer,
   int rows, cols;
   char hexbox_string[2] = {0, 0};
   double temp_x, temp_y;
-  PangoCairoHexBoxInfo *hbi;
+  PangoCairoFontHexBoxInfo *hbi;
   gunichar ch;
 
   cairo_save (crenderer->cr);
   cairo_get_current_point (crenderer->cr, &temp_x, &temp_y);
 
-  hbi = _pango_cairo_font_get_hex_box_info ((PangoCairoFont *)font);      
+  hbi = _pango_cairo_font_get_hex_box_info ((PangoCairoFont *)font);
 
-  if (!hbi || !_pango_cairo_font_install ((PangoCairoFont *) (hbi->font), crenderer->cr))
+  if (!hbi || !_pango_cairo_font_install ((PangoFont *)(hbi->font), crenderer->cr))
     {
       _pango_cairo_renderer_draw_box_glyph (crenderer, gi, cx, cy);
       goto done;
@@ -188,7 +178,7 @@ done:
   cairo_move_to (crenderer->cr, temp_x, temp_y);
   cairo_restore (crenderer->cr);
 }
-     
+
 static void
 pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
 				  PangoFont         *font,
@@ -200,17 +190,19 @@ pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
 
   /* cairo_glyph_t is 24 bytes */
 #define MAX_STACK 40
-  
+
   int i, count;
   int x_position = 0;
   cairo_glyph_t *cairo_glyphs;
   cairo_glyph_t stack_glyphs[MAX_STACK];
+  double base_x = crenderer->x_offset + (double)x / PANGO_SCALE;
+  double base_y = crenderer->y_offset + (double)y / PANGO_SCALE;
 
   cairo_save (crenderer->cr);
   if (!crenderer->do_path)
     set_color (crenderer, PANGO_RENDER_PART_FOREGROUND);
 
-  if (!_pango_cairo_font_install ((PangoCairoFont *) (font), crenderer->cr))
+  if (!_pango_cairo_font_install (font, crenderer->cr))
     {
       for (i = 0; i < glyphs->num_glyphs; i++)
 	{
@@ -218,13 +210,13 @@ pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
 
 	  if (gi->glyph != PANGO_GLYPH_EMPTY)
 	    {
-	      double cx = crenderer->x_offset +
-			  (double)(x + x_position + gi->geometry.x_offset) / PANGO_SCALE;
-	      double cy = crenderer->y_offset +
-			  (double)(y + gi->geometry.y_offset) / PANGO_SCALE;
+	      double cx = base_x + (double)(x_position + gi->geometry.x_offset) / PANGO_SCALE;
+	      double cy = gi->geometry.y_offset == 0 ?
+			  base_y :
+			  base_y + (double)(gi->geometry.y_offset) / PANGO_SCALE;
 
 	      _pango_cairo_renderer_draw_unknown_glyph (crenderer, font, gi, cx, cy);
-	    }	  
+	    }
 	  x_position += gi->geometry.width;
 	}
 
@@ -242,22 +234,22 @@ pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
       PangoGlyphInfo *gi = &glyphs->glyphs[i];
 
       if (gi->glyph != PANGO_GLYPH_EMPTY)
-        {
-          double cx = crenderer->x_offset +
-		      (double)(x + x_position + gi->geometry.x_offset) / PANGO_SCALE;
-          double cy = crenderer->y_offset +
-		      (double)(y + gi->geometry.y_offset) / PANGO_SCALE;
+	{
+	  double cx = base_x + (double)(x_position + gi->geometry.x_offset) / PANGO_SCALE;
+	  double cy = gi->geometry.y_offset == 0 ?
+		      base_y :
+		      base_y + (double)(gi->geometry.y_offset) / PANGO_SCALE;
 
-          if (gi->glyph & PANGO_GLYPH_UNKNOWN_FLAG)
+	  if (gi->glyph & PANGO_GLYPH_UNKNOWN_FLAG)
 	    _pango_cairo_renderer_draw_unknown_glyph (crenderer, font, gi, cx, cy);
-          else
-            {
-              cairo_glyphs[count].index = gi->glyph;
-              cairo_glyphs[count].x = cx;
-              cairo_glyphs[count].y = cy;
-              count++;
-            }
-        }	  
+	  else
+	    {
+	      cairo_glyphs[count].index = gi->glyph;
+	      cairo_glyphs[count].x = cx;
+	      cairo_glyphs[count].y = cy;
+	      count++;
+	    }
+	}
       x_position += gi->geometry.width;
     }
 
@@ -265,13 +257,13 @@ pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
     cairo_glyph_path (crenderer->cr, cairo_glyphs, count);
   else
     cairo_show_glyphs (crenderer->cr, cairo_glyphs, count);
-  
+
   if (glyphs->num_glyphs > MAX_STACK)
     g_free (cairo_glyphs);
 
 done:
   cairo_restore (crenderer->cr);
-  
+
 #undef MAX_STACK
 }
 
@@ -294,13 +286,13 @@ pango_cairo_renderer_draw_rectangle (PangoRenderer     *renderer,
 
   cairo_rectangle (crenderer->cr,
 		   crenderer->x_offset + (double)x / PANGO_SCALE,
-		   crenderer->y_offset + (double)y / PANGO_SCALE, 
+		   crenderer->y_offset + (double)y / PANGO_SCALE,
 		   (double)width / PANGO_SCALE, (double)height / PANGO_SCALE);
 
   if (!crenderer->do_path)
     {
       cairo_fill (crenderer->cr);
-    
+
       cairo_restore (crenderer->cr);
     }
 }
@@ -311,11 +303,11 @@ pango_cairo_renderer_draw_rectangle (PangoRenderer     *renderer,
  *   A/  \    /  \    /  \     A/  \    /  \              |
  *    \   \  /    \  /   /D     \   \  /    \             |
  *     \   \/  C   \/   /        \   \/   C  \            | height = HEIGHT_SQUARES * square
- *      \      /\  F   /          \  F   /\   \           | 
+ *      \      /\  F   /          \  F   /\   \           |
  *       \    /  \    /            \    /  \   \G         |
  *        \  /    \  /              \  /    \  /          |
  *         \/      \/                \/      \/           -
- *         B                         B       
+ *         B                         B
  *         |---|
  *       unit_width = (HEIGHT_SQUARES - 1) * square
  *
@@ -343,16 +335,16 @@ draw_error_underline (cairo_t *cr,
 
   y_top = y;
   y_bottom = y + height;
-  
+
   /* Bottom of squiggle */
   cairo_move_to (cr, x - square / 2, y_top + square / 2); /* A */
   for (i = 0; i < width_units; i += 2)
     {
       double x_middle = x + (i + 1) * unit_width;
       double x_right = x + (i + 2) * unit_width;
-    
+
       cairo_line_to (cr, x_middle, y_bottom); /* B */
-      
+
       if (i + 1 == width_units)
 	/* Nothing */;
       else if (i + 2 == width_units)
@@ -360,14 +352,14 @@ draw_error_underline (cairo_t *cr,
       else
 	cairo_line_to (cr, x_right, y_top + square); /* C */
     }
-  
+
   /* Top of squiggle */
   for (i -= 2; i >= 0; i -= 2)
     {
       double x_left = x + i * unit_width;
       double x_middle = x + (i + 1) * unit_width;
       double x_right = x + (i + 2) * unit_width;
-      
+
       if (i + 1 == width_units)
 	cairo_line_to (cr, x_middle + square / 2, y_bottom - square / 2); /* G */
       else {
@@ -375,7 +367,7 @@ draw_error_underline (cairo_t *cr,
 	  cairo_line_to (cr, x_right, y_top); /* E */
 	cairo_line_to (cr, x_middle, y_bottom - square); /* F */
       }
-      
+
       cairo_line_to (cr, x_left, y_top);   /* H */
     }
 }
@@ -395,7 +387,7 @@ pango_cairo_renderer_draw_error_underline (PangoRenderer *renderer,
       cairo_save (cr);
 
       set_color (crenderer, PANGO_RENDER_PART_UNDERLINE);
-      
+
       cairo_new_path (cr);
     }
 
@@ -403,7 +395,7 @@ pango_cairo_renderer_draw_error_underline (PangoRenderer *renderer,
 			crenderer->x_offset + (double)x / PANGO_SCALE,
 			crenderer->y_offset + (double)y / PANGO_SCALE,
 			(double)width / PANGO_SCALE, (double)height / PANGO_SCALE);
-  
+
   if (!crenderer->do_path)
     {
       cairo_fill (cr);
@@ -412,6 +404,44 @@ pango_cairo_renderer_draw_error_underline (PangoRenderer *renderer,
     }
 }
 
+static void
+pango_cairo_renderer_draw_shape (PangoRenderer  *renderer,
+				 PangoAttrShape *attr,
+				 int             x,
+				 int             y)
+{
+  PangoCairoRenderer *crenderer = (PangoCairoRenderer *) (renderer);
+  cairo_t *cr = crenderer->cr;
+  PangoLayout *layout;
+  PangoCairoShapeRendererFunc shape_renderer;
+  gpointer                    shape_renderer_data;
+  double base_x, base_y;
+
+  layout = pango_renderer_get_layout (renderer);
+
+  if (!layout)
+  	return;
+
+  shape_renderer = pango_cairo_context_get_shape_renderer (pango_layout_get_context (layout),
+							   &shape_renderer_data);
+
+  if (!shape_renderer)
+    return;
+
+  base_x = crenderer->x_offset + (double)x / PANGO_SCALE;
+  base_y = crenderer->y_offset + (double)y / PANGO_SCALE;
+
+  cairo_save (cr);
+  if (!crenderer->do_path)
+    set_color (crenderer, PANGO_RENDER_PART_FOREGROUND);
+
+  cairo_move_to (cr, base_x, base_y);
+
+  shape_renderer (cr, attr, crenderer->do_path, shape_renderer_data);
+
+  cairo_restore (cr);
+}
+
 static void
 pango_cairo_renderer_init (PangoCairoRenderer *renderer)
 {
@@ -425,22 +455,21 @@ pango_cairo_renderer_class_init (PangoCairoRendererClass *klass)
   renderer_class->draw_glyphs = pango_cairo_renderer_draw_glyphs;
   renderer_class->draw_rectangle = pango_cairo_renderer_draw_rectangle;
   renderer_class->draw_error_underline = pango_cairo_renderer_draw_error_underline;
+  renderer_class->draw_shape = pango_cairo_renderer_draw_shape;
 }
 
 static PangoCairoRenderer *cached_renderer = NULL;
-static GStaticMutex cached_renderer_mutex = G_STATIC_MUTEX_INIT;
+G_LOCK_DEFINE_STATIC (cached_renderer);
 
 static PangoCairoRenderer *
 acquire_renderer (gboolean *free_renderer)
 {
   PangoCairoRenderer *renderer;
 
-  /* renderer = _pango_cairo_font_map_get_renderer (PANGO_CAIRO_FONT_MAP (fontmap)); */
-
-  if (G_LIKELY (g_static_mutex_trylock (&cached_renderer_mutex)))
+  if (G_LIKELY (G_TRYLOCK (cached_renderer)))
     {
       if (G_UNLIKELY (!cached_renderer))
-        cached_renderer = g_object_new (PANGO_TYPE_CAIRO_RENDERER, NULL);
+	cached_renderer = g_object_new (PANGO_TYPE_CAIRO_RENDERER, NULL);
 
       renderer = cached_renderer;
       *free_renderer = FALSE;
@@ -464,7 +493,7 @@ release_renderer (PangoCairoRenderer *renderer, gboolean free_renderer)
       renderer->x_offset = 0.;
       renderer->y_offset = 0.;
 
-      g_static_mutex_unlock (&cached_renderer_mutex);
+      G_UNLOCK (cached_renderer);
     }
   else
     g_object_unref (renderer);
@@ -494,21 +523,21 @@ _pango_cairo_do_glyph_string (cairo_t          *cr,
        * prepare_run() isn't called.
        */
 
-      pango_renderer_activate (renderer); 	 
-	   	 
-      pango_renderer_set_color (renderer, PANGO_RENDER_PART_FOREGROUND, NULL); 	 
-      pango_renderer_set_color (renderer, PANGO_RENDER_PART_BACKGROUND, NULL); 	 
-      pango_renderer_set_color (renderer, PANGO_RENDER_PART_UNDERLINE, NULL); 	 
-      pango_renderer_set_color (renderer, PANGO_RENDER_PART_STRIKETHROUGH, NULL); 	 
+      pango_renderer_activate (renderer);
+
+      pango_renderer_set_color (renderer, PANGO_RENDER_PART_FOREGROUND, NULL);
+      pango_renderer_set_color (renderer, PANGO_RENDER_PART_BACKGROUND, NULL);
+      pango_renderer_set_color (renderer, PANGO_RENDER_PART_UNDERLINE, NULL);
+      pango_renderer_set_color (renderer, PANGO_RENDER_PART_STRIKETHROUGH, NULL);
     }
-  
+
   pango_renderer_draw_glyphs (renderer, font, glyphs, 0, 0);
 
   if (!do_path)
     {
       pango_renderer_deactivate (renderer);
     }
-  
+
   release_renderer (crenderer, free_renderer);
 }
 
@@ -526,7 +555,7 @@ _pango_cairo_do_layout_line (cairo_t          *cr,
   cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
 
   pango_renderer_draw_layout_line (renderer, line, 0, 0);
-  
+
   release_renderer (crenderer, free_renderer);
 }
 
@@ -538,24 +567,24 @@ _pango_cairo_do_layout (cairo_t     *cr,
   gboolean free_renderer;
   PangoCairoRenderer *crenderer = acquire_renderer (&free_renderer);
   PangoRenderer *renderer = (PangoRenderer *) crenderer;
-  
+
   crenderer->cr = cr;
   crenderer->do_path = do_path;
   cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
 
   pango_renderer_draw_layout (renderer, layout, 0, 0);
-  
+
   release_renderer (crenderer, free_renderer);
 }
 
-static void 
-_pango_cairo_do_error_underline (cairo_t *cr,                                  
+static void
+_pango_cairo_do_error_underline (cairo_t *cr,
 				 double   x,
 				 double   y,
 				 double   width,
 				 double   height,
 				 gboolean do_path)
-{  
+{
   /* We don't use a renderer here, for a simple reason:
    * the only renderer we can get is the default renderer, that
    * is all implemented here, so we shortcircuit and make our
@@ -569,7 +598,7 @@ _pango_cairo_do_error_underline (cairo_t *cr,
 
   if (!do_path)
     cairo_fill (cr);
-} 
+}
 
 
 /* public wrapper of above to show or append path */
@@ -578,9 +607,9 @@ _pango_cairo_do_error_underline (cairo_t *cr,
 /**
  * pango_cairo_show_glyph_string:
  * @cr: a Cairo context
- * @font: a #PangoFont
+ * @font: a #PangoFont from a #PangoCairoFontMap
  * @glyphs: a #PangoGlyphString
- * 
+ *
  * Draws the glyphs in @glyphs in the specified cairo context.
  * The origin of the glyphs (the left edge of the baseline) will
  * be drawn at the current point of the cairo context.
@@ -602,7 +631,7 @@ pango_cairo_show_glyph_string (cairo_t          *cr,
  * pango_cairo_show_layout_line:
  * @cr: a Cairo context
  * @line: a #PangoLayoutLine
- * 
+ *
  * Draws a #PangoLayoutLine in the specified cairo context.
  * The origin of the glyphs (the left edge of the line) will
  * be drawn at the current point of the cairo context.
@@ -623,7 +652,7 @@ pango_cairo_show_layout_line (cairo_t          *cr,
  * pango_cairo_show_layout:
  * @cr: a Cairo context
  * @layout: a Pango layout
- * 
+ *
  * Draws a #PangoLayoutLine in the specified cairo context.
  * The top-left corner of the #PangoLayout will be drawn
  * at the current point of the cairo context.
@@ -647,7 +676,7 @@ pango_cairo_show_layout (cairo_t     *cr,
  * @y: The Y coordinate of one corner of the rectangle
  * @width: Non-negative width of the rectangle
  * @height: Non-negative height of the rectangle
- * 
+ *
  * Draw a squiggly line in the specified cairo context that approximately
  * covers the given rectangle in the style of an underline used to indicate a
  * spelling error.  (The width of the underline is rounded to an integer
@@ -656,25 +685,25 @@ pango_cairo_show_layout (cairo_t     *cr,
  *
  * Since: 1.14
  **/
-void 
-pango_cairo_show_error_underline (cairo_t *cr,                                  
-			          double  x,
-			          double  y,
-			          double  width,
-			          double  height)
-{  
+void
+pango_cairo_show_error_underline (cairo_t *cr,
+				  double  x,
+				  double  y,
+				  double  width,
+				  double  height)
+{
   g_return_if_fail (cr != NULL);
-  g_return_if_fail ((width >= 0) && (height >= 0));  
-  
-  _pango_cairo_do_error_underline (cr, x, y, width, height, FALSE);                                 
-} 
+  g_return_if_fail ((width >= 0) && (height >= 0));
+
+  _pango_cairo_do_error_underline (cr, x, y, width, height, FALSE);
+}
 
 /**
  * pango_cairo_glyph_string_path
  * @cr: a Cairo context
- * @font: a #PangoFont
+ * @font: a #PangoFont from a #PangoCairoFontMap
  * @glyphs: a #PangoGlyphString
- * 
+ *
  * Adds the glyphs in @glyphs to the current path in the specified
  * cairo context. The origin of the glyphs (the left edge of the baseline)
  * will be at the current point of the cairo context.
@@ -696,7 +725,7 @@ pango_cairo_glyph_string_path (cairo_t          *cr,
  * pango_cairo_layout_line_path:
  * @cr: a Cairo context
  * @line: a #PangoLayoutLine
- * 
+ *
  * Adds the text in #PangoLayoutLine to the current path in the
  * specified cairo context.  The origin of the glyphs (the left edge
  * of the line) will be at the current point of the cairo context.
@@ -717,7 +746,7 @@ pango_cairo_layout_line_path (cairo_t          *cr,
  * pango_cairo_layout_path:
  * @cr: a Cairo context
  * @layout: a Pango layout
- * 
+ *
  * Adds the text in a #PangoLayout to the current path in the
  * specified cairo context.  The top-left corner of the #PangoLayout
  * will be at the current point of the cairo context.
@@ -741,7 +770,7 @@ pango_cairo_layout_path (cairo_t     *cr,
  * @y: The Y coordinate of one corner of the rectangle
  * @width: Non-negative width of the rectangle
  * @height: Non-negative height of the rectangle
- * 
+ *
  * Add a squiggly line to the current path in the specified cairo context that
  * approximately covers the given rectangle in the style of an underline used
  * to indicate a spelling error.  (The width of the underline is rounded to an
@@ -750,15 +779,15 @@ pango_cairo_layout_path (cairo_t     *cr,
  *
  * Since: 1.14
  **/
-void 
-pango_cairo_error_underline_path (cairo_t *cr,                                  
-			          double   x,
-			          double   y,
-			          double   width,
-			          double   height)
+void
+pango_cairo_error_underline_path (cairo_t *cr,
+				  double   x,
+				  double   y,
+				  double   width,
+				  double   height)
 {
   g_return_if_fail (cr != NULL);
   g_return_if_fail ((width >= 0) && (height >= 0));
-  
-  _pango_cairo_do_error_underline (cr, x, y, width, height, TRUE);                                 
+
+  _pango_cairo_do_error_underline (cr, x, y, width, height, TRUE);
 }
